import { Action } from '@ngrx/store';
import { User } from 'src/app/shared/models/user.model';

//type of actions
export enum UserActionType {
  /* action: get all users */
  GET_ALL_USERS = '[User] GET ALL USERS',
  GET_ALL_USERS_SUCCESS = '[User] GET-ALL USERS SUCCESS',
  GET_ALL_USERS_ERROR = '[User] GET-ALL USERS ERROR',

  /* action: search a user */

  SEARCH_A_USER = '[User] SEARCH A USER',
  SEARCH_A_USER_SUCCESS = '[User] SEARCH A USER SUCCESS',
  SEARCH_A_USER_ERROR = '[User] SEARCH A USER ERROR',

  /* action: edit a user */

  EDIT_A_USER = '[User] EDIT A USER',
  EDIT_A_USER_SUCCESS = '[User] EDIT A USER SUCCESS',
  EDIT_A_USER_ERROR = '[User] EDIT A USER ERROR',

  /* action of button: create a user form*/

  CREATE_A_USERFORM = '[User] CREATE USER FORM',
  CREATE_A_USERFORM_SUCCESS = '[User] CREATE A USER FORM SUCCESS',
  CREATE_A_USERFORM_ERROR = '[User] CREATE A USER FORM ERROR',

  SAVE_A_USERFORM ='[User] SAVE A USER',
  SAVE_A_USERFORM_SUCCESS ='[User] SAVE A USER SUCCESS',
  SAVE_A_USERFORM_ERROR='[User] SAVE A USER ERROR',

   /* action: delete a user */

   DELETE_A_USER = '[User] DELETE A USER',
   DELETE_A_USER_SUCCESS = '[User] DELETE A USER SUCCESS',
   DELETE_A_USER_ERROR = '[User] DELETE A USER ERROR'
}

//create actions

export class GetAllUsersAction implements Action {
  type: UserActionType = UserActionType.GET_ALL_USERS;

  constructor(public payload: any) {}
}

export class GetAllUsersActionSuccess implements Action {
  type: UserActionType = UserActionType.GET_ALL_USERS_SUCCESS;
  constructor(public payload: User[]) {}
}

export class GetAllUsersActionError implements Action {
  type: UserActionType = UserActionType.GET_ALL_USERS_ERROR;
  constructor(public payload: string) {}
}

/* Actios of searching a user */

export class SearchUsersAction implements Action {
  type: UserActionType = UserActionType.SEARCH_A_USER;

  constructor(public payload: any) {}
}

export class SearchUserActionSuccess implements Action {
  type: UserActionType = UserActionType.SEARCH_A_USER_SUCCESS;
  constructor(public payload: User[]) {}
}

export class SearchUserActionError implements Action {
  type: UserActionType = UserActionType.SEARCH_A_USER_ERROR;
  constructor(public payload: string) {}
}

/* Actios of editing a user */

export class EditUsersAction implements Action {
  type: UserActionType = UserActionType.EDIT_A_USER;

  constructor(public payload: {user: User}) {}
}

export class EditUserActionSuccess implements Action {
  type: UserActionType = UserActionType.EDIT_A_USER_SUCCESS;
  constructor(public payload: User) {}
}

export class EditUserActionError implements Action {
  type: UserActionType = UserActionType.EDIT_A_USER_ERROR;
  constructor(public payload: string) {}
}

/* Actios of creating a user form */

export class CreateUserFormAction implements Action {
  type: UserActionType = UserActionType.CREATE_A_USERFORM;

  constructor(public payload: any) {}
}

export class CreateUserFormActionSuccess implements Action {
  type: UserActionType = UserActionType.CREATE_A_USERFORM_SUCCESS;
  constructor(public payload: any) {}
}

export class CreateUserFormActionError implements Action {
  type: UserActionType = UserActionType.CREATE_A_USERFORM_ERROR;
  constructor(public payload: string) {}
}

/**action of saving a user */
export class SaveUserFormAction implements Action{
  type: UserActionType = UserActionType.SAVE_A_USERFORM;
  constructor(public payload: User){}
}

export class SaveUserFormActionSuccess implements Action{
  type: UserActionType = UserActionType.SAVE_A_USERFORM_SUCCESS;
  constructor(public payload: User){}
}

export class SaveUserFormActionError implements Action{
  type: UserActionType = UserActionType.SAVE_A_USERFORM_ERROR;
  constructor(public payload: string){}
}

/* Actios of deleting a user */

export class DeleteUsersAction implements Action {
  type: UserActionType = UserActionType.DELETE_A_USER;

  constructor(public payload: User) {}
}

export class DeleteUserActionSuccess implements Action {
  type: UserActionType = UserActionType.DELETE_A_USER_SUCCESS;
  constructor(public payload: User) {}
}

export class DeleteUserActionError implements Action {
  type: UserActionType = UserActionType.DELETE_A_USER_ERROR;
  constructor(public payload: string) {}
}

export type UserActionUnion =
  | GetAllUsersAction
  | GetAllUsersActionSuccess
  | GetAllUsersActionError
  | SearchUsersAction
  | SearchUserActionSuccess
  | SearchUserActionError
  | EditUsersAction
  | EditUserActionSuccess
  | EditUserActionError
  | CreateUserFormAction
  | CreateUserFormActionSuccess
  | CreateUserFormActionError
  | SaveUserFormAction
  | SaveUserFormActionSuccess
  | SaveUserFormActionError
  | DeleteUsersAction
  | DeleteUserActionSuccess
  | DeleteUserActionError;
