import { UserActionType, GetAllUsersActionSuccess, GetAllUsersActionError, UserActionUnion, SearchUserActionSuccess, SearchUserActionError, EditUserActionSuccess, EditUserActionError, DeleteUserActionSuccess, DeleteUserActionError, CreateUserFormActionSuccess, CreateUserFormActionError, SaveUserFormActionSuccess, SaveUserFormActionError } from './users.actions';
import { map, mergeMap, Observable, catchError, of } from 'rxjs';
import { UsersService } from './../../shared/services/users.service';
import { Injectable } from "@angular/core";
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { User } from 'src/app/shared/models/user.model';

@Injectable()
export class UserEffects{

  constructor(private usersService: UsersService, private effectActions: Actions){}

  //create getAllUsers effects

  getAllUsersEffect: Observable<UserActionUnion> = createEffect(

    () => this.effectActions.pipe(
      ofType(UserActionType.GET_ALL_USERS),
      mergeMap(
        () =>{

          return this.usersService.getAllUsers().pipe(
              map((users: User[])=> new GetAllUsersActionSuccess(users)),
              catchError((err) => of(new GetAllUsersActionError(err.message)))
            );
        }
      )

    )
  );
   //create searchUser effects
   searchUserEffect: Observable<UserActionUnion> = createEffect(

    () => this.effectActions.pipe(
      ofType(UserActionType.SEARCH_A_USER),
      mergeMap(
        (userAction: UserActionUnion) =>{

          return this.usersService.searchUser(userAction.payload).pipe(
              map((users: User[])=> new SearchUserActionSuccess(users)),
              catchError((err) => of(new SearchUserActionError(err.message)))
            );
        }
      )

    )
  );

  //create editUser effect
  editUserEffect: Observable<UserActionUnion> = createEffect(
    () => this.effectActions.pipe(
      ofType(UserActionType.EDIT_A_USER),
      mergeMap( (userAction: UserActionUnion)=>{
        return this.usersService.updateUser(userAction.payload).pipe(
          map( (user: User)=> new EditUserActionSuccess(user)),
          catchError( (err) => of(new EditUserActionError(err.message)))
        )
      })
    )
  );

  //create deleteUser effect
  deleteUserEffect: Observable<UserActionUnion> = createEffect(
    ()=> this.effectActions.pipe(
      ofType(UserActionType.DELETE_A_USER),
      mergeMap( (userAction: UserActionUnion) => {
        return this.usersService.deleteUser(userAction.payload.id).pipe(
          map( ()=> new DeleteUserActionSuccess(userAction.payload)),
          catchError( (err) => of(new DeleteUserActionError(err.message)))
        )
      })
    )
  );

  //create createUser effect: action of button to create user form
  createUserFormEffect: Observable<UserActionUnion> = createEffect(
    () => this.effectActions.pipe(
      ofType(UserActionType.CREATE_A_USERFORM),
      map( ()=>{
        return new CreateUserFormActionSuccess({});
      }),
      catchError( (err) => of(new CreateUserFormActionError(err.message)))
    )
  );

  //create save a user effect: action of saving data of the form
  saveUserFormEffect : Observable<UserActionUnion> = createEffect(
    () => this.effectActions.pipe(
      ofType(UserActionType.SAVE_A_USERFORM),
      mergeMap( (userAction: UserActionUnion) => {
        return this.usersService.createUser(userAction.payload).pipe(
          map (user => new SaveUserFormActionSuccess(user)),
          catchError( err => of(new SaveUserFormActionError(err.message)))
        );
      })
    )
  );
}
