import { Action } from '@ngrx/store';
import { User } from 'src/app/shared/models/user.model';
import { MsgState } from '../msg.state';
import { UserActionUnion, UserActionType } from './users.actions';

//create a state
export class UserState {
  usersList!: User[];
  msgState!: MsgState;
  userServerErrorMsg!: string;
}

//create reducer, a reducer need a initial state and an action
const initUserState: UserState = {
  usersList: [],
  msgState: MsgState.INIT_STATE,
  userServerErrorMsg: 'No error message',
};

export function userReducer(
  state: UserState = initUserState,
  action: Action
): UserState {
  switch (action.type) {
    /** get all users actions */
    case UserActionType.GET_ALL_USERS:
      return { ...state, msgState: MsgState.LOADING };

    case UserActionType.GET_ALL_USERS_SUCCESS:
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: (<UserActionUnion>action).payload,
      };

    case UserActionType.GET_ALL_USERS_ERROR:
      return {
        ...state,
        msgState: MsgState.ERROR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    /** search a user actions */
    case UserActionType.SEARCH_A_USER:
      return { ...state, msgState: MsgState.LOADING };

    case UserActionType.SEARCH_A_USER_SUCCESS:
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: (<UserActionUnion>action).payload,
      };

    case UserActionType.SEARCH_A_USER_ERROR:
      return {
        ...state,
        msgState: MsgState.ERROR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    /** edit a user actions */
    case UserActionType.EDIT_A_USER:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };

    case UserActionType.EDIT_A_USER_SUCCESS:
      let modifiedUser: User = (<UserActionUnion>action).payload;
      let usersListCopy1: User[] = [...state.usersList];
      let usersListNew1: User[] = usersListCopy1.map((user) =>
        user.id == modifiedUser.id ? modifiedUser : user
      );
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: usersListNew1,
      };

    case UserActionType.EDIT_A_USER_ERROR:
      return {
        ...state,
        msgState: MsgState.ERROR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    /** create a user form actions */
    case UserActionType.CREATE_A_USERFORM:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };

    case UserActionType.CREATE_A_USERFORM_SUCCESS:
      return {
        ...state,
        msgState: MsgState.NEW,
      };

    case UserActionType.CREATE_A_USERFORM_ERROR:
      return {
        ...state,
        msgState: MsgState.ERROR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    /** save a user (data of form) action*/
    case UserActionType.SAVE_A_USERFORM:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };
    case UserActionType.CREATE_A_USERFORM_SUCCESS:
      let createdUser: User = (<UserActionUnion>action).payload;
      let usersListCopy2: User[] = [...state.usersList];
      usersListCopy2.push(createdUser);

      return {
        ...state,
        msgState: MsgState.SUCCESS,
        usersList: usersListCopy2,
      };

    /** delete a user actions */
    case UserActionType.DELETE_A_USER:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };

    case UserActionType.DELETE_A_USER_SUCCESS:
      let deletedUser: User = (<UserActionUnion>action).payload;
      let deletedUserIndex = state.usersList.indexOf(deletedUser);
      let usersListNew3: User[] = [...state.usersList];
      usersListNew3.splice(deletedUserIndex, 1);
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: usersListNew3,
      };

    case UserActionType.DELETE_A_USER_ERROR:
      return {
        ...state,
        msgState: MsgState.ERROR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    default:
      return { ...state };
  }
}
