import { HttpErrorResponse, HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { catchError, Observable, throwError } from 'rxjs';
import { environment } from 'src/environments/environment';
import { Company } from '../models/company.model';

@Injectable({
  providedIn: 'root'
})
export class CompaniesService {

  constructor(private http: HttpClient) { }

  handleError(err: HttpErrorResponse) {
    if (err.status === 0) {
      console.error('error has occurred:', err.error);
    } else {
      console.error(
        'Backend returned code ${error.status}, body was:',
        err.error
      );
    }

    return throwError(() => {
      new Error('something went wrong, try again');
    });
  }

  baseUrl: string = environment.host;
  headers = { 'content-type': 'application/json' };

  getAllCompanies(): Observable<Company[]> {
    return this.http.get<Company[]>(this.baseUrl + '/companies');
  }

  getAllProjectsOfCompany(): Observable<Company[]> {
    return this.http
      .get<Company[]>(this.baseUrl + '/companies?_embed=projects')
      .pipe(catchError(this.handleError));
  }

  createCompany(company: Company): Observable<Company> {
    return this.http
      .post<Company>(this.baseUrl + '/companies', company, { headers: this.headers })
      .pipe(catchError(this.handleError));
  }

  deleteCompany(companyId: number) {
    return this.http
      .delete(this.baseUrl + '/companies/' + companyId, {
        headers: this.headers,
      })
      .pipe(catchError(this.handleError));
  }

  getCompanyById(companyId: number): Observable<Company> {
    return this.http.get<Company>(this.baseUrl +'/companies/'+companyId);
  }

  getCompanyByCompanyName(companyName: string): Observable<Company[]> {
    return this.http.get<Company[]>(this.baseUrl +'/companies?companyName='+companyName);
  }

  updateCompany(company: Company): Observable<Company> {
    return this.http.put<Company>(this.baseUrl + '/companies/'+ company.id, company, {
      headers: this.headers,
    }).pipe(catchError(this.handleError));
  }
}
