import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Injectable, OnInit } from '@angular/core';
import { catchError, Observable, retry, throwError } from 'rxjs';
import { environment } from 'src/environments/environment';
import { Project } from '../models/project.model';

@Injectable({
  providedIn: 'root',
})
export class ProjectsService implements OnInit {
  constructor(private http: HttpClient) {}
  ngOnInit(): void {}

  handleError(err: HttpErrorResponse) {
    if (err.status === 0) {
      console.error('error has occurred:', err.error);
    } else {
      console.error(
        'Backend returned code ${error.status}, body was:',
        err.error
      );
    }

    return throwError(() => {
      new Error('something went wrong, try again');
    });
  }

  baseUrl: string = environment.host;
  headers = { 'content-type': 'application/json' };

  getAllProjects(): Observable<Project[]> {
    return this.http.get<Project[]>(this.baseUrl + '/projects');
  }

  getAllProjectsWithUsersAsociated(): Observable<Project[]> {
    return this.http
      .get<Project[]>(this.baseUrl + '/projects?_expand=user')
      .pipe(catchError(this.handleError));
  }

  createProject(project: Project): Observable<Project> {
    return this.http
      .post<Project>(this.baseUrl + '/projects', project, { headers: this.headers })
      .pipe(catchError(this.handleError));
  }

  deleteProject(projectId: number) {
    return this.http
      .delete(this.baseUrl + '/projects/' + projectId, {
        headers: this.headers,
      })
      .pipe(catchError(this.handleError));
  }

  getProjectById(projectId: number): Observable<Project> {
    return this.http.get<Project>(this.baseUrl +'/projects/'+projectId);
  }

  getProjectByName(projectName: string): Observable<Project[]> {
    return this.http.get<Project[]>(this.baseUrl +'/projects?projectName='+projectName);
  }

  updateProject(project: Project): Observable<Project> {
    return this.http.put<Project>(this.baseUrl + '/projects/'+ project.id, project, {
      headers: this.headers,
    }).pipe(catchError(this.handleError));
  }

  getProjectWithCompanyAssociated():Observable<any>{
    return this.http.get<any>(this.baseUrl+'/projects?_expand=company')
    .pipe(catchError(this.handleError))
  }
}
